@@ -9,46 +9,20 @@
 
 #include <KAboutData>
 #include <KCrash>
+#include <QApplication>
 #include <QCommandLineParser>
 
-#include <KAuthorized>
 #include <KDBusService>
+#include <KLocalizedString>
+#include <KPluginMetaData>
 #include <KQuickAddons/QtQuickSettings>
-#include <KServiceTypeTrader>
 #include <KWindowSystem>
 #include <iostream>
 #include <kworkspace.h>
 
+#include "../core/kcmmetadatahelpers.h"
 #include "SettingsBase.h"
 
-KService::List m_modules;
-
-static bool caseInsensitiveLessThan(const KService::Ptr s1, const KService::Ptr s2)
-{
-    const int compare = QString::compare(s1->desktopEntryName(), s2->desktopEntryName(), Qt::CaseInsensitive);
-    return (compare < 0);
-}
-
-static void listModules()
-{
-    // First condition is what systemsettings does, second what kinfocenter does, make sure this is kept in sync
-    // We need the exist calls because otherwise the trader language aborts if the property doesn't exist and the second part of the or is not evaluated
-    KService::List services =
-        KServiceTypeTrader::self()->query(QStringLiteral("KCModule"),
-                                          QStringLiteral("(exist [X-KDE-System-Settings-Parent-Category] and [X-KDE-System-Settings-Parent-Category] != '') or "
-                                                         "(exist [X-KDE-ParentApp] and [X-KDE-ParentApp] == 'kinfocenter')"));
-    services += KServiceTypeTrader::self()->query(QStringLiteral("SystemSettingsExternalApp"));
-    for (KService::List::const_iterator it = services.constBegin(); it != services.constEnd(); ++it) {
-        const KService::Ptr s = (*it);
-        if (!KAuthorized::authorizeControlModule(s->menuId())) {
-            continue;
-        }
-        m_modules.append(s);
-    }
-
-    std::stable_sort(m_modules.begin(), m_modules.end(), caseInsensitiveLessThan);
-}
-
 int main(int argc, char *argv[])
 {
     // Make sure the binary name is either kinfocenter or systemsettings,
@@ -61,7 +35,7 @@ int main(int argc, char *argv[])
         mode = BaseMode::InfoCenter;
     }
 
-    // exec is systemsettings5, but we need the QPT to use the right config from the qApp constructor
+    // exec is systemsettings, but we need the QPT to use the right config from the qApp constructor
     // which is before KAboutData::setApplicationData
     QCoreApplication::setApplicationName(binaryName);
 
@@ -80,7 +54,7 @@ int main(int argc, char *argv[])
         // About data
         aboutData = KAboutData(QStringLiteral("kinfocenter"),
                                i18n("Info Center"),
-                               QLatin1String(PROJECT_VERSION),
+                               QStringLiteral(PROJECT_VERSION),
                                i18n("Centralized and convenient overview of system information."),
                                KAboutLicense::GPL,
                                i18n("(c) 2009, Ben Cooksley"));
@@ -91,7 +65,7 @@ int main(int argc, char *argv[])
     } else {
         aboutData = KAboutData(QStringLiteral("systemsettings"),
                                i18n("System Settings"),
-                               QLatin1String(PROJECT_VERSION),
+                               QStringLiteral(PROJECT_VERSION),
                                i18n("Central configuration center by KDE."),
                                KAboutLicense::GPL,
                                i18n("(c) 2009, Ben Cooksley"));
@@ -117,7 +91,7 @@ int main(int argc, char *argv[])
 
     parser.addOption(QCommandLineOption(QStringLiteral("list"), i18n("List all possible modules")));
     parser.addPositionalArgument(QStringLiteral("module"), i18n("Configuration module to open"));
-    parser.addOption(QCommandLineOption(QStringLiteral("args"), i18n("Arguments for the module"), QLatin1String("arguments")));
+    parser.addOption(QCommandLineOption(QStringLiteral("args"), i18n("Arguments for the module"), QStringLiteral("arguments")));
 
     aboutData.setupCommandLine(&parser);
 
@@ -127,22 +101,23 @@ int main(int argc, char *argv[])
     if (parser.isSet(QStringLiteral("list"))) {
         std::cout << i18n("The following modules are available:").toLocal8Bit().data() << std::endl;
 
-        listModules();
+        auto source = mode == BaseMode::InfoCenter ? MetaDataSource::KInfoCenter : MetaDataSource::SystemSettings;
+        const auto modules = findKCMsMetaData(source) << findExternalKCMModules(source);
 
         int maxLen = 0;
 
-        for (KService::List::ConstIterator it = m_modules.constBegin(); it != m_modules.constEnd(); ++it) {
-            int len = (*it)->desktopEntryName().length();
+        for (const auto &metaData : modules) {
+            const int len = metaData.pluginId().length();
             if (len > maxLen) {
                 maxLen = len;
             }
         }
 
-        for (KService::List::ConstIterator it = m_modules.constBegin(); it != m_modules.constEnd(); ++it) {
+        for (const auto &metaData : modules) {
             QString entry(QStringLiteral("%1 - %2"));
 
-            entry = entry.arg((*it)->desktopEntryName().leftJustified(maxLen, QLatin1Char(' ')))
-                        .arg(!(*it)->comment().isEmpty() ? (*it)->comment() : i18n("No description available"));
+            entry = entry.arg(metaData.pluginId().leftJustified(maxLen, QLatin1Char(' ')))
+                        .arg(!metaData.description().isEmpty() ? metaData.description() : i18n("No description available"));
 
             std::cout << entry.toLocal8Bit().data() << std::endl;
         }
@@ -158,7 +133,7 @@ int main(int argc, char *argv[])
     QString startupModule;
 
     if (parser.positionalArguments().count() == 1) {
-        startupModule = parser.positionalArguments().first();
+        startupModule = parser.positionalArguments().constFirst();
     }
 
     if (!args.isEmpty() && startupModule.isEmpty()) {
@@ -180,7 +155,7 @@ int main(int argc, char *argv[])
         // We don't need to do the error checking again though.
         QCommandLineParser parser;
         parser.addPositionalArgument(QStringLiteral("module"), i18n("Configuration module to open"));
-        parser.addOption(QCommandLineOption(QStringLiteral("args"), i18n("Arguments for the module"), QLatin1String("arguments")));
+        parser.addOption(QCommandLineOption(QStringLiteral("args"), i18n("Arguments for the module"), QStringLiteral("arguments")));
 
         parser.parse(arguments);
 
@@ -188,7 +163,7 @@ int main(int argc, char *argv[])
         QString startupModule;
 
         if (parser.positionalArguments().count() == 1) {
-            startupModule = parser.positionalArguments().first();
+            startupModule = parser.positionalArguments().constFirst();
         }
 
         if (!startupModule.isEmpty()) {
